-------------------------------------------------------------
-- MSS copyright 2011-2012
--	Filename:  HIGH_SPEED_NCO_SUBSTITUTE.VHD
-- Authors: 
--		Alain Zarembowitch / MSS, 
--	Version: 1
--	Date last modified: 5/12/12
-- Inheritance: 	SYMBOL_CLKGEN.vhd 5/10/12
--
-- description: Substitute for HIGH_SPEED_NCO.vhd using a fixed PLL. Interface compatible. 
-- Not programmable at runtime.
-- Change DIVCLK_DIVIDE, M,D parameters in PLL to change frequency.
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
library unisim;
use unisim.vcomponents.all;

entity HIGH_SPEED_NCO is
port
 (
  CLK60G: in     std_logic;
	-- input reference clock global. 60 MHz (PLL valid input range 19 - 375 MHz)
  NCO_FREQ: in std_logic_vector(19 downto 0);
	-- ignored
  CLK_OUTG          : out    std_logic;
	-- Clock out ports (after BUFGs)
  LOCKED_PLL            : out    std_logic
	-- pll lock status 
 );
end entity;

architecture Behavioral of HIGH_SPEED_NCO is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLKFB1: std_logic;
signal PLL_CLKOUT0: std_logic;
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin


-- first PLL generates a fixed 300 MHz from the 60 MHz reference
PLL_BASE_001 : PLL_BASE 
generic map(
 BANDWIDTH            => "LOW",	-- fixed PLL, prefer low jitter to fast lock
 CLK_FEEDBACK         => "CLKFBOUT",
 COMPENSATION         => "INTERNAL",
-- DIVCLK_DIVIDE        => 2,	
-- CLKFBOUT_MULT        => 24,	-- 720 MHz = (VCO / M) = (60 MHz / D) input reference freq.
-- CLKOUT0_DIVIDE       => 6,	-- 120 MHz
--	DIVCLK_DIVIDE        => 2,	
--	CLKFBOUT_MULT        => 20,	-- 600 MHz = (VCO / M) = (60 MHz / D) input reference freq.
--	CLKOUT0_DIVIDE       => 6,	-- 100 MHz
 DIVCLK_DIVIDE        => 1,	
 CLKFBOUT_MULT        => 8,	-- 480 MHz = (VCO / M) = (60 MHz / D) input reference freq.
 CLKOUT0_DIVIDE       => 6,	-- 80 MHz
 CLKFBOUT_PHASE       => 0.000,
 CLKOUT0_PHASE        => 0.0,
 CLKOUT0_DUTY_CYCLE   => 0.500,
 CLKIN_PERIOD         => 16.666,
 REF_JITTER           => 0.010)
port map
 -- Output clocks
(CLKFBOUT            => CLKFB1,
 CLKOUT0             => PLL_CLKOUT0,
 -- Status and control signals
 LOCKED              => LOCKED_PLL,
 RST                 => '0',
 -- Input clock control
 CLKFBIN             => CLKFB1,
 CLKIN               => CLK60G
 );


-- make 300 MHz CLK0 global
BUFG_001: BUFG port map(I => PLL_CLKOUT0, O => CLK_OUTG);


end Behavioral;
